@@ -17,19 +17,26 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+type ModbusWorkarounds struct {
+	PollPause        config.Duration `toml:"pause_between_requests"`
+	CloseAfterGather bool            `toml:"close_connection_after_gather"`
+}
+
 // Modbus holds all data relevant to the plugin
 type Modbus struct {
-	Name             string          `toml:"name"`
-	Controller       string          `toml:"controller"`
-	TransmissionMode string          `toml:"transmission_mode"`
-	BaudRate         int             `toml:"baud_rate"`
-	DataBits         int             `toml:"data_bits"`
-	Parity           string          `toml:"parity"`
-	StopBits         int             `toml:"stop_bits"`
-	Timeout          config.Duration `toml:"timeout"`
-	Retries          int             `toml:"busy_retries"`
-	RetriesWaitTime  config.Duration `toml:"busy_retries_wait"`
-	Log              telegraf.Logger `toml:"-"`
+	Name             string            `toml:"name"`
+	Controller       string            `toml:"controller"`
+	TransmissionMode string            `toml:"transmission_mode"`
+	BaudRate         int               `toml:"baud_rate"`
+	DataBits         int               `toml:"data_bits"`
+	Parity           string            `toml:"parity"`
+	StopBits         int               `toml:"stop_bits"`
+	Timeout          config.Duration   `toml:"timeout"`
+	Retries          int               `toml:"busy_retries"`
+	RetriesWaitTime  config.Duration   `toml:"busy_retries_wait"`
+	DebugConnection  bool              `toml:"debug_connection"`
+	Workarounds      ModbusWorkarounds `toml:"workarounds"`
+	Log              telegraf.Logger   `toml:"-"`
 	// Register configuration
 	ConfigurationOriginal
 	// Connection handling
@@ -90,19 +97,24 @@ const sampleConfig = `
 
   # TCP - connect via Modbus/TCP
   controller = "tcp://localhost:502"
- 
+
   ## Serial (RS485; RS232)
   # controller = "file:///dev/ttyUSB0"
   # baud_rate = 9600
   # data_bits = 8
   # parity = "N"
   # stop_bits = 1
+  # transmission_mode = "RTU"
+
+  ## Trace the connection to the modbus device as debug messages
+  ## Note: You have to enable telegraf's debug mode to see those messages!
+  # debug_connection = false
 
   ## For Modbus over TCP you can choose between "TCP", "RTUoverTCP" and "ASCIIoverTCP"
   ## default behaviour is "TCP" if the controller is TCP
   ## For Serial you can choose between "RTU" and "ASCII"
   # transmission_mode = "RTU"
- 
+
   ## Measurements
   ##
 
@@ -150,6 +162,15 @@ const sampleConfig = `
     { name = "tank_ph",      byte_order = "AB",   data_type = "INT16",   scale=1.0,     address = [1]},
     { name = "pump1_speed",  byte_order = "ABCD", data_type = "INT32",   scale=1.0,     address = [3,4]},
   ]
+
+  ## Enable workarounds required by some devices to work correctly
+  # [inputs.modbus.workarounds]
+    ## Pause between read requests sent to the device. This might be necessary for (slow) serial devices.
+    # pause_between_requests = "0ms"
+    ## Close the connection after every gather cycle. Usually the plugin closes the connection after a certain
+    ## idle-timeout, however, if you query a device with limited simultaneous connectivity (e.g. serial devices)
+    ## from multiple instances you might want to only stay connected during gather and disconnect afterwards.
+    # close_connection_after_gather = false
 `
 
 // SampleConfig returns a basic configuration for the plugin
@@ -236,6 +257,11 @@ func (m *Modbus) Gather(acc telegraf.Accumulator) error {
 		m.collectFields(acc, timestamp, tags, requests.input)
 	}
 
+	// Disconnect after read if configured
+	if m.Workarounds.CloseAfterGather {
+		return m.disconnect()
+	}
+
 	return nil
 }
 
@@ -255,14 +281,23 @@ func (m *Modbus) initClient() error {
 		case "RTUoverTCP":
 			handler := mb.NewRTUOverTCPClientHandler(host + ":" + port)
 			handler.Timeout = time.Duration(m.Timeout)
+			if m.DebugConnection {
+				handler.Logger = m
+			}
 			m.handler = handler
 		case "ASCIIoverTCP":
 			handler := mb.NewASCIIOverTCPClientHandler(host + ":" + port)
 			handler.Timeout = time.Duration(m.Timeout)
+			if m.DebugConnection {
+				handler.Logger = m
+			}
 			m.handler = handler
 		default:
 			handler := mb.NewTCPClientHandler(host + ":" + port)
 			handler.Timeout = time.Duration(m.Timeout)
+			if m.DebugConnection {
+				handler.Logger = m
+			}
 			m.handler = handler
 		}
 	case "file":
@@ -274,6 +309,9 @@ func (m *Modbus) initClient() error {
 			handler.DataBits = m.DataBits
 			handler.Parity = m.Parity
 			handler.StopBits = m.StopBits
+			if m.DebugConnection {
+				handler.Logger = m
+			}
 			m.handler = handler
 		case "ASCII":
 			handler := mb.NewASCIIClientHandler(u.Path)
@@ -282,6 +320,9 @@ func (m *Modbus) initClient() error {
 			handler.DataBits = m.DataBits
 			handler.Parity = m.Parity
 			handler.StopBits = m.StopBits
+			if m.DebugConnection {
+				handler.Logger = m
+			}
 			m.handler = handler
 		default:
 			return fmt.Errorf("invalid protocol '%s' - '%s' ", u.Scheme, m.TransmissionMode)
@@ -336,6 +377,7 @@ func (m *Modbus) gatherRequestsCoil(requests []request) error {
 		if err != nil {
 			return err
 		}
+		nextRequest := time.Now().Add(time.Duration(m.Workarounds.PollPause))
 		m.Log.Debugf("got coil@%v[%v]: %v", request.address, request.length, bytes)
 
 		// Bit value handling
@@ -347,6 +389,9 @@ func (m *Modbus) gatherRequestsCoil(requests []request) error {
 			request.fields[i].value = uint16((bytes[idx] >> bit) & 0x01)
 			m.Log.Debugf("  field %s with bit %d @ byte %d: %v --> %v", field.name, bit, idx, (bytes[idx]>>bit)&0x01, request.fields[i].value)
 		}
+
+		// Some (serial) devices require a pause between requests...
+		time.Sleep(time.Until(nextRequest))
 	}
 	return nil
 }
@@ -358,6 +403,7 @@ func (m *Modbus) gatherRequestsDiscrete(requests []request) error {
 		if err != nil {
 			return err
 		}
+		nextRequest := time.Now().Add(time.Duration(m.Workarounds.PollPause))
 		m.Log.Debugf("got discrete@%v[%v]: %v", request.address, request.length, bytes)
 
 		// Bit value handling
@@ -369,6 +415,9 @@ func (m *Modbus) gatherRequestsDiscrete(requests []request) error {
 			request.fields[i].value = uint16((bytes[idx] >> bit) & 0x01)
 			m.Log.Debugf("  field %s with bit %d @ byte %d: %v --> %v", field.name, bit, idx, (bytes[idx]>>bit)&0x01, request.fields[i].value)
 		}
+
+		// Some (serial) devices require a pause between requests...
+		time.Sleep(time.Until(nextRequest))
 	}
 	return nil
 }
@@ -380,6 +429,7 @@ func (m *Modbus) gatherRequestsHolding(requests []request) error {
 		if err != nil {
 			return err
 		}
+		nextRequest := time.Now().Add(time.Duration(m.Workarounds.PollPause))
 		m.Log.Debugf("got holding@%v[%v]: %v", request.address, request.length, bytes)
 
 		// Non-bit value handling
@@ -392,6 +442,9 @@ func (m *Modbus) gatherRequestsHolding(requests []request) error {
 			request.fields[i].value = field.converter(bytes[offset : offset+length])
 			m.Log.Debugf("  field %s with offset %d with len %d: %v --> %v", field.name, offset, length, bytes[offset:offset+length], request.fields[i].value)
 		}
+
+		// Some (serial) devices require a pause between requests...
+		time.Sleep(time.Until(nextRequest))
 	}
 	return nil
 }
@@ -403,6 +456,7 @@ func (m *Modbus) gatherRequestsInput(requests []request) error {
 		if err != nil {
 			return err
 		}
+		nextRequest := time.Now().Add(time.Duration(m.Workarounds.PollPause))
 		m.Log.Debugf("got input@%v[%v]: %v", request.address, request.length, bytes)
 
 		// Non-bit value handling
@@ -415,6 +469,9 @@ func (m *Modbus) gatherRequestsInput(requests []request) error {
 			request.fields[i].value = field.converter(bytes[offset : offset+length])
 			m.Log.Debugf("  field %s with offset %d with len %d: %v --> %v", field.name, offset, length, bytes[offset:offset+length], request.fields[i].value)
 		}
+
+		// Some (serial) devices require a pause between requests...
+		time.Sleep(time.Until(nextRequest))
 	}
 	return nil
 }
@@ -443,6 +500,11 @@ func (m *Modbus) collectFields(acc telegraf.Accumulator, timestamp time.Time, ta
 	}
 }
 
+// Implement the logger interface of the modbus client
+func (m *Modbus) Printf(format string, v ...interface{}) {
+	m.Log.Debugf(format, v...)
+}
+
 // Add this plugin to telegraf
 func init() {
 	inputs.Add("modbus", func() telegraf.Input { return &Modbus{} })
